
//Chris' WinGlide enhancements
//Copr. 1998, Chris Dohnal (cwdohnal@ucdavis.edu)

#include <windows.h>
#include <glide.h>
#include "pointers.h"

extern int sw, sh;

//Functions from dib.c
extern void dibinit(HWND hwnd, int w, int h);
extern void dibquit();
extern short *dibgetbuf();
extern void dibupdate();
extern int dibgetpitch();

VOID CWGInitialize(HWND, int, int);
VOID CWGShutdown(VOID);
BOOL CWGCopy3DfxFrameBuffer(VOID);

VOID CWGInitialize(HWND hWnd, int iWidth, int iHeight) {
	
	//Create the DIB
	dibinit(hWnd, iWidth, iHeight);
	
	return;
}

VOID CWGShutdown(VOID) {
		
	//Delete the DIB
	dibquit();

	return;
}

BOOL CWGCopy3DfxFrameBuffer(VOID) {
	FxBool fxbRet;
	GrLfbInfo_t grLFBInfo;
	long lLine;
	LPVOID p3DfxScanlineStart;
	int swDivFour;
	short *pDIB;
	int iDIBPitch;

	//Get information about the DIB
	pDIB = dibgetbuf();
	iDIBPitch = dibgetpitch();
	if (pDIB == NULL) {
		return FALSE;
	}

	//Lock the 3Dfx LFB
	ZeroMemory(&grLFBInfo, sizeof(grLFBInfo));
	grLFBInfo.size = sizeof(grLFBInfo);
	fxbRet = _grLfbLock(GR_LFB_READ_ONLY, GR_BUFFER_BACKBUFFER, 0,
		GR_ORIGIN_ANY, FXFALSE, &grLFBInfo);
	if (fxbRet != FXTRUE) {
		//Could not lock the 3Dfx LFB
		return FALSE;
	}

	//Setup pointers to the 3Dfx LFB and the DIB
	p3DfxScanlineStart = (BYTE *)grLFBInfo.lfbPtr +
		grLFBInfo.strideInBytes * (sh - 1);


	//Transfer the data from the 3Dfx LFB to the DIB
	swDivFour = sw >> 2;
	for (lLine = 0; lLine < sh; lLine++) {
		//Transfer a scanline
		_asm {
			mov esi, p3DfxScanlineStart
			mov edi, pDIB
			mov ecx, swDivFour
			
			xor edx, edx
		copyloop:
			movq mm0, qword ptr [esi + edx*8]
			movq qword ptr [edi + edx*8], mm0
			inc edx
			dec ecx
			jnz copyloop			
		}
		
		//Calculate the address of the next scanline
		(BYTE *)p3DfxScanlineStart -= grLFBInfo.strideInBytes;
		(BYTE *)pDIB += iDIBPitch;
	}
	//Emtpy the MMX state
	_asm emms

	//Unlock the 3Dfx LFB
	fxbRet = _grLfbUnlock(GR_LFB_READ_ONLY, GR_BUFFER_BACKBUFFER);
	if (fxbRet != FXTRUE) {
		//Could not unlock the 3Dfx LFB
		return FALSE;
	}

	//Put the DIB on the screen
	dibupdate();

	return TRUE;
}
