// main.c


// Contains the .dll entry point and overridden functions
// uses imports.c for loading symbols, dib.c for getting framebuffer access.

// Copr. 1998 - Justin Frankel/Nullsoft, Inc. (http://www.nullsoft.com)

// Version 0.5 - 1/4/1998

//Some modifications made by Chris Dohnal (cwdohnal@ucdavis.edu)

#include <windows.h>

#define FX_GLIDE_NO_FUNC_PROTO
#include <glide.h>
#include <glideutl.h>

#include "pointers.h"
#include "cwg.h"

HINSTANCE hGlide, hDllInst;

#define ENTRY __declspec(dllexport)
#define CALL __stdcall

// imports.c functions
extern int LoadSymbols(HINSTANCE *hGlide, HINSTANCE hDllInst);

// DLL entry point. Calls on load/unload of the .dll. Loads or frees the real glide2x.dll.
BOOL WINAPI DllMain(HINSTANCE hInstDll, DWORD fdwReason, LPVOID lpvReserved) {

	hDllInst = hInstDll;
	
	if (fdwReason == DLL_PROCESS_ATTACH) {
		if (!LoadSymbols(&hGlide,hDllInst)) {
			hGlide = NULL;
			return FALSE;
		}
	}

	if (fdwReason == DLL_PROCESS_DETACH) {
		FreeModule(hGlide);
		hGlide = NULL;
	}
	return TRUE;
}

int enable=1, // are we running windowed?
    sw, // screen width (window width)
	sh; // screen height (window height)
// quake 2 320x240 and 400x300 hack.
int iQuake2LowResOffset;


// Our new, overridden functions go here. 


// override grSstWinOpen, for opening the 3dfx hardware.
ENTRY FxBool CALL grSstWinOpen(FxU32 hFxWnd, GrScreenResolution_t grResolution,
	GrScreenRefresh_t grRefresh, GrColorFormat_t grColorFormat,
	GrOriginLocation_t grOriginLocation, int nColBuffers, int nAuxBuffers) {
	
	HWND hWnd;
	
	//Get hFxWnd as a HWND
	hWnd = (HWND)hFxWnd;

	if (_grSstWinOpen(hFxWnd, grResolution, grRefresh, grColorFormat,
		grOriginLocation, nColBuffers, nAuxBuffers) == FXFALSE) {
		
		return FXFALSE;
	}

	sw = _grSstScreenWidth();
	sh = _grSstScreenHeight();
	
	// decide if we are running fullscreen or windowed
	if (hWnd && IsWindow(hWnd)) {
		RECT rect;
		GetWindowRect(hWnd, &rect);
		if ((rect.left == 0) && (rect.top == 0) && (rect.right == sw) &&
			(rect.bottom == sh)) {
			
			enable = 0;
		}
		else {
			enable = 1;
		}
	} 
	else {
		enable = 0;
	}
	
	// if enabled, initialize us
	if (enable) {
		RECT rect;

		GetClientRect(hWnd, &rect);

		if (grOriginLocation == GR_ORIGIN_LOWER_LEFT && sh == 384) {
			iQuake2LowResOffset = 384 - rect.bottom;
		}
		else {
			iQuake2LowResOffset = 0;
		}

		sw = rect.right;
		sh = rect.bottom;
		_grSstControl(GR_CONTROL_DEACTIVATE);
		_grRenderBuffer(GR_BUFFER_BACKBUFFER);

		//Call CWG initialization function
		CWGInitialize(hWnd, sw, sh);
	}
	return FXTRUE;
}

// override grBufferSwap to copy the hardware's data into our DIBSection
ENTRY void CALL grBufferSwap(int swapInterval) {
	BOOL bRet;
	
	if (enable) {
		bRet = CWGCopy3DfxFrameBuffer();
		if (bRet == FALSE) {
			return;
		}
	}
	else {
		_grBufferSwap(swapInterval);
	}
	return;
}

// override grSstWinClose to delete our DIB
ENTRY void CALL grSstWinClose(void) {
	_grSstWinClose();
	if (enable) {
		//Call CWG shutdown function
		CWGShutdown();
	}
	return;
}

// override grSstControl to keep the calling app from reenabling the hardware's video output
ENTRY FxBool CALL grSstControl(FxU32 code) {
	if (!enable) {
		return _grSstControl(code);
	}
	return 0;
}
